@@ -1,5 +1,67 @@
+##' Function minimization with box-constraints
+##' 
+##' This Davidon-Fletcher-Powell optimization algorithm has been `hand-tuned'
+##' for minimal setup configuration and for efficency. It uses an internal
+##' logit-type transformation based on the pre-specified box-constraints.
+##' Therefore, it usually does not require rescaling (see help for the R optim
+##' function). \code{dfp} automatically computes step sizes for each parameter
+##' to operate with sufficient sensitivity in the functional output.
+##' Performance is comparable to the BFGS algorithm in the R function
+##' \code{optim}. \code{dfp} interfaces with \code{newton} to ascertain
+##' convergence, compute the eigenvalues of the Hessian, and provide 95\%
+##' confidence intervals when the function to be minimized is a negative
+##' log-likelihood.
+##' 
+##' The dfp function minimizes a function \code{f} over the parameters
+##' specified in the input list \code{x}. The algorithm is based on Fletcher's
+##' "Switching Method" (Comp.J. 13,317 (1970))
+##' 
+##' the code has been 'transcribed' from Fortran source code into R
+##' 
+##' @param x a list with components 'label' (of mode character), 'est' (the
+##' parameter vector with the initial guess), 'low' (vector with lower bounds),
+##' and 'upp' (vector with upper bounds)
+##' @param f the function that is to be minimized over the parameter vector
+##' defined by the list \code{x}
+##' @param tol a tolerance used to determine when convergence should be
+##' indicated
+##' @param nfcn number of function calls
+##' @param ... other parameters to be passed to `f`
+##' @return list with the following components: \item{fmin }{ the function
+##' value f at the minimum } \item{label }{ the labels taken from list \code{x}
+##' } \item{est }{ a vector of the estimates at the minimum. dfp does not
+##' overwrite \code{x} } \item{status }{ 0 indicates convergence, 1 indicates
+##' non-convergence } \item{nfcn }{ no. of function calls }
+##' @note This function is part of the Bhat exploration tool
+##' @author E. Georg Luebeck (FHCRC)
+##' @seealso optim, \code{\link{newton}}, \code{\link{ftrf}},
+##' \code{\link{btrf}}, \code{\link{logit.hessian}}
+##' @references Fletcher's Switching Method (Comp.J. 13,317, 1970)
+##' @keywords optimize methods
+##' @examples
+##' 
+##'         # generate some Poisson counts on the fly
+##'           dose <- c(rep(0,50),rep(1,50),rep(5,50),rep(10,50))
+##'           data <- cbind(dose,rpois(200,20*(1+dose*.5*(1-dose*0.05))))
+##' 
+##'         # neg. log-likelihood of Poisson model with 'linear-quadratic' mean: 
+##'           lkh <- function (x) { 
+##'           ds <- data[, 1]
+##'           y  <- data[, 2]
+##'           g <- x[1] * (1 + ds * x[2] * (1 - x[3] * ds)) 
+##'           return(sum(g - y * log(g)))
+##'           }
+##' 
+##' 	# for example define
+##'           x <- list(label=c("a","b","c"),est=c(10.,10.,.01),low=c(0,0,0),upp=c(100,20,.1))
+##' 
+##' 	# call:
+##' 	  results <- dfp(x,f=lkh)
+##' 
+##' @export
+##' 
 "dfp" <-
-function (x, f, tol=1e-5, nfcn = 0) 
+function (x, f, tol=1e-5, nfcn = 0, ...) 
 {
 	    #     Function Minimization for R. 
 	    #     This function is part of the Bhat exploration tool and is
@@ -16,6 +78,8 @@ function (x, f, tol=1e-5, nfcn = 0)
 
             #     E Georg Luebeck (gluebeck@fhcrc.org)
 
+            ff = function(x) f(x, ...) ## 1-arg function (closure) to be optimized
+    
             xt.inf <- 16
 	    slamin <- 0.2
 	    slamax <- 3
@@ -48,7 +112,7 @@ function (x, f, tol=1e-5, nfcn = 0)
 	    ####  first call 
 	    cat(date(), "\n","\n")
 	    xt <- ftrf(x$est, x$low, x$upp)
-	    fmin <- f(x$est)
+	    fmin <- ff(x$est)
 	    nfcn <- nfcn + 1
             cat('starting at','\n')
 	    cat(format(nfcn), "  fmin: ", fmin, "   ", format(x$est), "\n")
@@ -73,7 +137,7 @@ function (x, f, tol=1e-5, nfcn = 0)
 	    	    #### 1, 2 
 	    	    #### ?? cat("COVARIANCE MATRIX NOT POSITIVE-DEFINITE","\n")
 	    	    #### define step sizes dirin
-                    d <- dqstep(list(label=x$label,est=btrf(xt, x$low, x$upp),low=x$low,upp=x$upp),f,sens=.01)
+                    d <- dqstep(list(label=x$label,est=btrf(xt, x$low, x$upp),low=x$low,upp=x$upp),ff,sens=.01)
 	    	    if (isw2 >= 1)  d <- 0.02 * sqrt(abs(diag(v)) * up)
 	    	    dirin <- d
 
@@ -89,10 +153,10 @@ function (x, f, tol=1e-5, nfcn = 0)
 	    	    	    	    d <- dirin[i]
 	    	    	    	    xtf <- xt[i]
 	    	    	    	    xt[i] <- xtf + d
-	    	    	    	    fs1 <- f(btrf(xt, x$low, x$upp))
+	    	    	    	    fs1 <- ff(btrf(xt, x$low, x$upp))
 	    	    	    	    nfcn <- nfcn + 1
 	    	    	    	    xt[i] <- xtf - d
-	    	    	    	    fs2 <- f(btrf(xt, x$low, x$upp))
+	    	    	    	    fs2 <- ff(btrf(xt, x$low, x$upp))
 	    	    	    	    nfcn <- nfcn + 1
 	    	    	    	    xt[i] <- xtf
 	    	    	    	    gs[i] <- (fs1 - fs2)/(2 * d)
@@ -115,7 +179,7 @@ function (x, f, tol=1e-5, nfcn = 0)
 	    	    	    	     # while(ns < 10 ...)
 	    	    	    	     while (ns < 10 & nf < 10) {
 	    	    	    	      xt[i] <- xtf + d
-	    	    	    	      f0 <- f(btrf(xt, x$low, x$upp))
+	    	    	    	      f0 <- ff(btrf(xt, x$low, x$upp))
 	    	    	    	      nfcn <- nfcn + 1
 	    	    	    	      #       cat("dfp search intermediate output:","\n")
 	    	    	    	      #       cat("f0: ",f0,"  fmin: ",fmin,"   nfcn: ",nfcn,"\n")
@@ -235,7 +299,7 @@ function (x, f, tol=1e-5, nfcn = 0)
 	    	    	    xt <- xxs + dirin
 	    	    	    xt[xt > xt.inf] <- xt.inf
 	    	    	    xt[xt < -xt.inf] <- -xt.inf
-	    	    	    f0 <- f(btrf(xt, x$low, x$upp))
+	    	    	    f0 <- ff(btrf(xt, x$low, x$upp))
 	    	    	    nfcn <- nfcn + 1
 	    	    	    ####  cat(format(nfcn),"   f0: ",f0,"   ",format(xt),"\n","\n")
 	    	    	    ####  change to output on orig. scale
@@ -261,7 +325,7 @@ function (x, f, tol=1e-5, nfcn = 0)
 	    	    	    	    xt <- xxs + slam * dirin
 	    	    	    	    xt[xt > xt.inf] <- xt.inf
 	    	    	    	    xt[xt < -xt.inf] <- -xt.inf
-	    	    	    	    f2 <- f(btrf(xt, x$low, x$upp))
+	    	    	    	    f2 <- ff(btrf(xt, x$low, x$upp))
 	    	    	    	    nfcn <- nfcn + 1
 	    	    	    	    ####   cat(format(nfcn),"  f2: ",f2,"   ",format(xt),"\n")
 	    	    	    	    ####   quadr interp using 3 points
@@ -286,7 +350,7 @@ function (x, f, tol=1e-5, nfcn = 0)
 	    	    	    	    xt <- xxs + tlam * dirin
 	    	    	    	    xt[xt > xt.inf] <- xt.inf
 	    	    	    	    xt[xt < -xt.inf] <- -xt.inf
-	    	    	    	    f3 <- f(btrf(xt, x$low, x$upp))
+	    	    	    	    f3 <- ff(btrf(xt, x$low, x$upp))
 	    	    	    	    nfcn <- nfcn + 1
 	    	    	    	    if (f0 >= fmin & f2 >= fmin & f3 >= fmin) {
 	    	    	    	     f.main <- 200
@@ -334,7 +398,7 @@ function (x, f, tol=1e-5, nfcn = 0)
 	    	    	    iter <- iter + 1
 	    	    	    ####  get gradient and sigma
 	    	    	    ####  compute first and second (diagonal) derivatives 
-	    	    	    fmin <- f(btrf(xt, x$low, x$upp))
+	    	    	    fmin <- ff(btrf(xt, x$low, x$upp))
 	    	    	    nfcn <- nfcn + 1
 	    	    	    ####      cat(format(nfcn),"  ",fmin,"   ",format(xt),"\n")
 	    	    	    ####      cat("___________________________________________","\n")
@@ -348,10 +412,10 @@ function (x, f, tol=1e-5, nfcn = 0)
 	    	    	    	    }
 	    	    	    	    xtf <- xt[i]
 	    	    	    	    xt[i] <- xtf + d
-	    	    	    	    fs1 <- f(btrf(xt, x$low, x$upp))
+	    	    	    	    fs1 <- ff(btrf(xt, x$low, x$upp))
 	    	    	    	    nfcn <- nfcn + 1
 	    	    	    	    xt[i] <- xtf - d
-	    	    	    	    fs2 <- f(btrf(xt, x$low, x$upp))
+	    	    	    	    fs2 <- ff(btrf(xt, x$low, x$upp))
 	    	    	    	    nfcn <- nfcn + 1
 	    	    	    	    xt[i] <- xtf
 	    	    	    	    g[i] <- (fs1 - fs2)/(2 * d)
@@ -457,12 +521,12 @@ function (x, f, tol=1e-5, nfcn = 0)
 	    	    	    break
 	    	    }
                   }
-	    fmin <- f(btrf(xt, x$low, x$upp)); nfcn <- nfcn + 1
+	    fmin <- ff(btrf(xt, x$low, x$upp)); nfcn <- nfcn + 1
 
 	    x$est <- btrf(xt, x$low, x$upp)
             ####  compute error (logit scale)
-            del <- dqstep(x,f,sens=.01)
-            h <- logit.hessian(x,f,del,dapprox=FALSE,nfcn); nfcn <- h$nfcn
+            del <- dqstep(x,ff,sens=.01)
+            h <- logit.hessian(x,ff,del,dapprox=FALSE,nfcn); nfcn <- h$nfcn
             v <- solve(h$ddf)
             
             xtl <- xt-1.96*sqrt(diag(v))
@@ -480,7 +544,7 @@ function (x, f, tol=1e-5, nfcn = 0)
 	    dimnames(m.out) <- list(1:npar, c("label", "estimate", "low", "high"))
 	    print(m.out, quote = FALSE)
 	    cat("\n")
-	    return(list(fmin = fmin, label = x$label, est = x$est, status=status, nfcn=nfcn))
+	    return(list(fmin = fmin, label = x$label, est = x$est, low=xl, high=xu, status=status, nfcn=nfcn))
 }
 
 
